@@ -17,7 +17,7 @@
 #'        vector to apply other different functions to numeric and categorical
 #'        \code{x}, where factors are first converted to numeric vectors, e.g.
 #'        \code{fun = c(numeric = "median", factor = "mean")}. See 'Examples'.
-#' @param weight.by Name of variable in \code{x} that indicated the vector of
+#' @param weights Name of variable in \code{x} that indicated the vector of
 #'   weights that will be applied to weight all observations. Default is
 #'   \code{NULL}, so no weights are used.
 #' @param ... Further arguments, passed down to \code{fun}.
@@ -51,8 +51,8 @@
 #' wt <- c(5,  5,  4,  1) / 15
 #' x <- c(3.7, 3.3, 3.5, 2.8)
 #'
-#' typical_value(x, "weighted.mean")
-#' typical_value(x, "weighted.mean", weight.by = wt)
+#' typical_value(x, fun = "weighted.mean")
+#' typical_value(x, fun = "weighted.mean", weights = wt)
 #'
 #' # for factors, return either reference level or mode value
 #' set.seed(123)
@@ -65,14 +65,7 @@
 #'
 #'
 #' @export
-typical_value <- function(x, fun = "mean", weight.by = NULL, ...) {
-
-  ## TODO activate later
-
-  # if (!missing(weight.by)) {
-  #   # message("Argument `weight.by` is deprecated. Please use `weights`.")
-  #   weights <- weight.by
-  # }
+typical_value <- function(x, fun = "mean", weights = NULL, ...) {
 
   # check if we have named vectors and find the requested function
   # for special functions for factors, convert to numeric first
@@ -95,11 +88,11 @@ typical_value <- function(x, fun = "mean", weight.by = NULL, ...) {
 
   # for weighted mean, check that weights are of same length as x
 
-  if (fun == "weighted.mean" && !is.null(weight.by)) {
+  if (fun == "weighted.mean" && !is.null(weights)) {
 
     # make sure weights and x have same length
 
-    if (length(weight.by) != length(x)) {
+    if (length(weights) != length(x)) {
       # if not, tell user and change function to mean
       warning("Vector of weights is of different length than `x`. Using `mean` as function for typical value.", call. = F)
       fun <- "mean"
@@ -108,7 +101,7 @@ typical_value <- function(x, fun = "mean", weight.by = NULL, ...) {
 
     # make sure weights are differen from 1
 
-    if (all(weight.by == 1)) {
+    if (all(weights == 1)) {
       # if not, tell user and change function to mean
       warning("All weight values are `1`. Using `mean` as function for typical value.", call. = F)
       fun <- "mean"
@@ -118,7 +111,7 @@ typical_value <- function(x, fun = "mean", weight.by = NULL, ...) {
 
   # no weights, than use normal mean function
 
-  if (fun == "weighted.mean" && is.null(weight.by)) fun <- "mean"
+  if (fun == "weighted.mean" && is.null(weights)) fun <- "mean"
 
 
   if (fun == "median")
@@ -134,7 +127,7 @@ typical_value <- function(x, fun = "mean", weight.by = NULL, ...) {
 
   if (is.numeric(x)) {
     if (fun == "weighted.mean")
-      do.call(myfun, args = list(x = x, na.rm = TRUE, w = weight.by, ...))
+      do.call(myfun, args = list(x = x, na.rm = TRUE, w = weights, ...))
     else
       do.call(myfun, args = list(x = x, na.rm = TRUE, ...))
   } else if (is.factor(x)) {
